"use strict";

exports.__esModule = true;
exports.buildGroupState = exports.buildGroups = exports.buildFooterValues = exports.filterAndSortData = exports.initializeFilters = exports.normalizePrimaryProperty = exports.datumValue = void 0;

// This file contains helper functions for DataTable, to keep the component
// files simpler.
// get the value for the property in the datum object
var datumValue = function datumValue(datum, property) {
  var parts = property.split('.');

  if (parts.length === 1) {
    return datum[property];
  }

  if (!datum[parts[0]]) {
    return undefined;
  }

  return datumValue(datum[parts[0]], parts.slice(1).join('.'));
}; // get the primary property name


exports.datumValue = datumValue;

var normalizePrimaryProperty = function normalizePrimaryProperty(columns, primaryKey) {
  var result;
  columns.forEach(function (column) {
    // remember the first key property
    if (column.primary && !result) {
      result = column.property;
    }
  });

  if (!result && columns.length > 0) {
    result = primaryKey || columns[0].property;
  }

  return result;
}; // initialize filters with empty strings


exports.normalizePrimaryProperty = normalizePrimaryProperty;

var initializeFilters = function initializeFilters(columns) {
  var result = {};
  columns.forEach(function (column) {
    if (column.search) {
      result[column.property] = '';
    }
  });
  return result;
}; // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions#Escaping


exports.initializeFilters = initializeFilters;

var escapeRegExp = function escapeRegExp(input) {
  return input.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
}; // filter data based on filters then sort


var filterAndSortData = function filterAndSortData(data, filters, onSearch, sort) {
  var result = data;

  if (!onSearch) {
    var regexps = {};
    Object.keys(filters).filter(function (n) {
      return filters[n];
    }).forEach(function (n) {
      regexps[n] = new RegExp(escapeRegExp(filters[n]), 'i');
    });

    if (Object.keys(regexps).length > 0) {
      result = data.filter(function (datum) {
        return !Object.keys(regexps).some(function (property) {
          return !regexps[property].test(datumValue(datum, property));
        });
      });
    }
  }

  if (sort) {
    var property = sort.property,
        ascending = sort.ascending;
    result = result === data ? [].concat(data) : result; // don't sort caller's data

    var before = ascending ? 1 : -1;
    var after = ascending ? -1 : 1;
    result.sort(function (d1, d2) {
      if (d1[property] > d2[property]) return before;
      if (d1[property] < d2[property]) return after;
      return 0;
    });
  }

  return result;
}; // aggregate reducers


exports.filterAndSortData = filterAndSortData;

var sumReducer = function sumReducer(accumulated, next) {
  return accumulated + next;
};

var minReducer = function minReducer(accumulated, next) {
  return accumulated === undefined ? next : Math.min(accumulated, next);
};

var maxReducer = function maxReducer(accumulated, next) {
  return accumulated === undefined ? next : Math.max(accumulated, next);
};

var reducers = {
  max: maxReducer,
  min: minReducer,
  sum: sumReducer
}; // aggregate a single column

var aggregateColumn = function aggregateColumn(column, data) {
  var value;

  if (column.aggregate === 'avg') {
    value = data.map(function (d) {
      return datumValue(d, column.property);
    }).reduce(sumReducer);
    value /= data.length;
  } else {
    value = data.map(function (d) {
      return datumValue(d, column.property);
    }).reduce(reducers[column.aggregate], 0);
  }

  return value;
}; // aggregate all columns that can


var aggregate = function aggregate(columns, data) {
  var result = {};
  columns.forEach(function (column) {
    if (column.aggregate) {
      result[column.property] = aggregateColumn(column, data);
    }
  });
  return result;
}; // build the values for the footer cells


var buildFooterValues = function buildFooterValues(columns, data) {
  var aggregateValues = aggregate(columns, data);
  var result = {};
  columns.forEach(function (column) {
    if (column.footer) {
      if (typeof column.footer === 'string') {
        result[column.property] = column.footer;
      } else if (column.footer.aggregate) {
        result[column.property] = aggregateValues[column.property];
      }
    }
  });
  return result;
}; // looks at the groupBy property of each data object and returns an
// array with one item for each unique value of that property.


exports.buildFooterValues = buildFooterValues;

var buildGroups = function buildGroups(columns, data, groupBy) {
  var result;

  if (groupBy) {
    result = [];
    var groupMap = {};
    data.forEach(function (datum) {
      var groupByProperty = groupBy.property ? groupBy.property : groupBy;
      var groupValue = datumValue(datum, groupByProperty);

      if (!groupMap[groupValue]) {
        var group = {
          data: [],
          datum: {},
          key: groupValue
        };
        group.datum[groupByProperty] = groupValue;
        result.push(group);
        groupMap[groupValue] = group;
      }

      groupMap[groupValue].data.push(datum);
    }); // include any aggregate column values across the data for each group

    columns.forEach(function (column) {
      if (column.aggregate) {
        result.forEach(function (group) {
          var datum = group.datum;
          datum[column.property] = aggregateColumn(column, group.data);
        });
      }
    });
  }

  return result;
}; // build group expanded state, expanding any in groupBy.expand


exports.buildGroups = buildGroups;

var buildGroupState = function buildGroupState(groups, groupBy) {
  var result = {};

  if (groups) {
    groups.forEach(function (_ref) {
      var key = _ref.key;
      result[key] = {
        expanded: false
      };
    });
  }

  if (groupBy && groupBy.expand) {
    groupBy.expand.forEach(function (value) {
      result[value] = {
        expanded: true
      };
    });
  }

  return result;
};

exports.buildGroupState = buildGroupState;